package org.glassfish.deployment.client;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;
import javax.enterprise.deploy.shared.ModuleType;
import javax.enterprise.deploy.shared.factories.DeploymentFactoryManager;
import javax.enterprise.deploy.spi.DeploymentManager;
import javax.enterprise.deploy.spi.TargetModuleID;
import javax.enterprise.deploy.spi.exceptions.DeploymentManagerCreationException;
import javax.enterprise.deploy.spi.exceptions.TargetException;
import javax.enterprise.deploy.spi.factories.DeploymentFactory;
import javax.enterprise.deploy.spi.status.ProgressEvent;
import javax.enterprise.deploy.spi.status.ProgressListener;
import javax.enterprise.deploy.spi.status.ProgressObject;
import org.glassfish.deployapi.DeploymentStatusImplWithError;

/*
 * 
 * Example taken from http://blogs.oracle.com/japod/entry/using_jsr_88_for_web
 *
 */
public class JSR88Deployer {

    class DeploymentListener implements ProgressListener {

        JSR88Deployer driver;
        String warContext;

        DeploymentListener(JSR88Deployer driver, String warContext) {
            this.driver = driver;
            if (warContext.contains("/")
                    && warContext.lastIndexOf('/') != warContext.length() - 1) {
                this.warContext = warContext.substring(warContext.lastIndexOf('/') + 1);
            } else {
                this.warContext = warContext;
            }
        }

        public void handleProgressEvent(ProgressEvent event) {
            if (event.getDeploymentStatus().isCompleted()) {
                System.out.println("DeploymentListener: completed - " + event.getDeploymentStatus().getMessage());
                try {
                    TargetModuleID[] ids = getDeploymentManager().getNonRunningModules(ModuleType.WAR, getDeploymentManager().getTargets());
                    TargetModuleID[] myIDs = new TargetModuleID[1];
                    for (TargetModuleID id : ids) {
                        if (warContext.equals(id.getModuleID())) {
                            myIDs[0] = id;
                            ProgressObject startProgress = driver.getDeploymentManager().start(myIDs);
                            startProgress.addProgressListener(new ProgressListener() {

                                public void handleProgressEvent(ProgressEvent event) {
                                    System.out.println(event.getDeploymentStatus().getMessage());
                                    if (event.getDeploymentStatus().isCompleted()) {
                                        driver.setAppStarted(true);
                                    }
                                }
                            });
                        }
                    }
                } catch (IllegalStateException ex) {
                    ex.printStackTrace();
                } catch (TargetException ex) {
                    ex.printStackTrace();
                }
            } else if (event.getDeploymentStatus().isFailed()) {
                System.out.println("DeploymentListener: failed");
                Object o = event.getDeploymentStatus();
                if (o instanceof DeploymentStatusImplWithError) {
                    ((DeploymentStatusImplWithError) event.getDeploymentStatus()).getCause().printStackTrace();
                } else {
                    System.out.println(event.getDeploymentStatus().getMessage());
                }
                // stuck here if appStarted is not set to true
                driver.setAppStarted(true);
            } else if (event.getDeploymentStatus().isRunning()) {
                System.out.println("DeploymentListener: running");
            }
        }
    }
    private final static String SyntaxHelp = "syntax:\\n\\tdeploy \\n\\tundeploy ";
    private static String PropertiesFilename = "wardeployment.properties";
    private Properties deploymentProperties;
    DeploymentManager deploymentManager;
    boolean appStarted;
    boolean appUndeployed;
    String warContext;
    String warFilename;
    String wsdlUrl;

    public JSR88Deployer() {
    }

    public JSR88Deployer(String filename) {
        setProperties(filename);
    }

    private void setProperties(String filename) {
        FileInputStream fis = null;
        try {
            fis = new FileInputStream(filename);
            deploymentProperties = new Properties();
            deploymentProperties.load(fis);
            fis.close();
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    synchronized void setAppStarted(boolean appStarted) {
        this.appStarted = appStarted;
        notifyAll();
    }

    synchronized void setAppUndeployed(boolean appUndeployed) {
        this.appUndeployed = appUndeployed;
        notifyAll();
    }

    private String getParam(String param) {
        return (null == deploymentProperties) ? null : deploymentProperties.getProperty(param);
    }

    public DeploymentManager getDeploymentManager() {
        if (null == deploymentManager) {
            DeploymentFactoryManager dfm = DeploymentFactoryManager.getInstance();
            try {
                Class dfClass = Class.forName(getParam("jsr88.df.classname"));
                DeploymentFactory dfInstance;
                dfInstance = (DeploymentFactory) dfClass.newInstance();
                dfm.registerDeploymentFactory(dfInstance);
            } catch (ClassNotFoundException ex) {
                ex.printStackTrace();
            } catch (IllegalAccessException ex) {
                ex.printStackTrace();
            } catch (InstantiationException ex) {
                ex.printStackTrace();
            }
            try {
                deploymentManager =
                        dfm.getDeploymentManager(
                        getParam("jsr88.dm.id"), getParam("jsr88.dm.user"), getParam("jsr88.dm.passwd"));

            } catch (DeploymentManagerCreationException ex) {
                ex.printStackTrace();
            }
        }
        return deploymentManager;
    }

    void runApp(String warFilename, String warContext) {
        setAppStarted(false);

        ProgressObject deplProgress = getDeploymentManager().distribute(getDeploymentManager().getTargets(),
                new File(warFilename), null);
        deplProgress.addProgressListener(new DeploymentListener(this, warContext));
        waitForAppStart();
    }

    void undeployApp(String warContext) {
        setAppUndeployed(false);
        try {
            TargetModuleID[] ids = getDeploymentManager().getAvailableModules(ModuleType.WAR, getDeploymentManager().getTargets());
            TargetModuleID[] myIDs = new TargetModuleID[1];
            for (TargetModuleID id : ids) {
                if (warContext.equals(id.getModuleID())) {
                    myIDs[0] = id;
                    ProgressObject startProgress = getDeploymentManager().undeploy(myIDs);
                    startProgress.addProgressListener(new ProgressListener() {

                        public void handleProgressEvent(ProgressEvent event) {
                            System.out.println(event.getDeploymentStatus().getMessage());

                            if (event.getDeploymentStatus().isCompleted()) {
                                setAppUndeployed(true);
                            }
                        }
                    });
                }
            }

        } catch (IllegalStateException ex) {
            ex.printStackTrace();
        } catch (TargetException ex) {
            ex.printStackTrace();
        }
        waitForAppUndeployment();
    }

    void releaseDeploymentManager() {
        if (null != deploymentManager) {
            deploymentManager.release();
        }
    }

    synchronized void waitForAppStart() {
        while (!appStarted) {
            try {
                wait();
            } catch (InterruptedException e) {
            }
        }
    }

    synchronized void waitForAppUndeployment() {
        while (!appUndeployed) {
            try {
                wait();
            } catch (InterruptedException e) {
            }
        }
    }

    private static void printHelpAndExit() {
        System.out.println(SyntaxHelp);
        System.exit(1);
    }

    public static void main(String[] args) throws ClassNotFoundException {
        if (args.length < 1) {
            printHelpAndExit();
        }
        if ("deploy".equals(args[0])) {
            String appName;
            if (args.length < 4) {
                appName = args[1].substring(0, args[1].length() - 4);
                if (args.length == 3) {
                    PropertiesFilename = args[2];
                }
            } else {
                appName = args[2];
                if (args.length == 4) {
                    PropertiesFilename = args[3];
                }
            }
            JSR88Deployer worker = new JSR88Deployer(PropertiesFilename);
            System.out.println("Deploying app...");
            worker.runApp(args[1], appName);
            worker.releaseDeploymentManager();
        } else if ("undeploy".equals(args[0])) {
            if (args.length == 3) {
                PropertiesFilename = args[2];
            }
            JSR88Deployer worker = new JSR88Deployer(PropertiesFilename);
            System.out.println("Undeploying app...");
            worker.undeployApp(args[1]);
            worker.releaseDeploymentManager();
        }
    }
}